;;
;; texts.scm
;;
;; Textual content.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit texts))

(module
 texts
 (
  banner-line
  banner
  license
  )

 (import scheme
	 (chicken base)
	 (chicken string)
	 (chicken format))

 ;; Short banner
 (define banner-line "HackerBase 1.16-dev (c) 2023 Brmlab, z.s.")

 ;; Banner source with numbers for ANSI CSI SGR
 (define banner-source "
1    __  __           __            
1   / / / /___ ______/ /_____  _____ 
2  / /_/ / __ `/ ___/ //_/ _ \\/ ___/ Space
3 / __  / /_/ / /__/ ,< /  __/ / 0____                
0/_/ /_/\\__,_/\\___/_/|_|\\___/_/ 0/ __ )____ _________ 
                              7/ __  / __ `/ ___/ _ \\
     5M 4E 5M 4B 5E 4R 5S      6Data / /_/ / /_/ (__  )  __/
                            1/_____/\\__,_/____/\\___/ 
")

 ;; Mapping of color tags to ANSI CSI SGR
 (define colors
   '((#\0 . 90)				; W
     (#\1 . 97)
     (#\2 . 91)				; R
     (#\3 . 31)
     (#\4 . 92)				; G
     (#\5 . 32)
     (#\6 . 94)				; B
     (#\7 . 34)))

 ;; Fill-in ANSI CSI SGR to produce printable banner
 (define banner
   (string-intersperse
    (map (lambda (line)
	   (string-append
	    (list->string
	     (flatten
	      (map (lambda (ch)
		     (let ((color (assq ch colors)))
		       (if color
			   (string->list (sprintf "\x1b[~Am" (cdr color)))
			   ch)))
		   (string->list line))))
	    "\x1b[0m"))
	 (string-split banner-source "\n"))
    "\n"))

 ;; The license of this file and of the whole suite.
 (define license "ISC License

Copyright 2023 Brmlab, z.s.
Dominik Pantůček <dominik.pantucek@trustica.cz>

Permission to use, copy, modify, and/or distribute this software
for any purpose with or without fee is hereby granted, provided
that the above copyright notice and this permission notice appear
in all copies.

THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
")

 )
