;;
;; tiocgwinsz.scm
;;
;; Get size of current terminal.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit tiocgwinsz))

(import duck)

(foreign-declare "#include <sys/ioctl.h>")

(module*
 tiocgwinsz
 #:doc ("TTY terminal size support.")
 (
  term-size
  )

 (import scheme
	 (chicken foreign)
	 (chicken bitwise))

 (define tiocgwinsz-ioctl
   (foreign-lambda*
    int ()
    "
struct winsize wss;
if (ioctl(0, TIOCGWINSZ, &wss) == 0) {
  C_return(wss.ws_row*65536+wss.ws_col);
} else {
  C_return(0);
}
"
    ))

 (define/doc (term-size)
   ("Returns the number of terminal rows and columns.")
   (let ((res (tiocgwinsz-ioctl)))
     (values 
      (arithmetic-shift res -16)
      (bitwise-and res #xffff))))

 )
